"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const Endpoints_1 = require("../utils/Endpoints");
const Methods_1 = require("../utils/Methods");
const BaseClient_1 = require("./BaseClient");
const ErrorCodes_1 = require("../errors/ErrorCodes");
const iFunnyError_1 = __importDefault(require("../errors/iFunnyError"));
/**
 * Client for the Discord API.
 * @extends BaseClient
 */
class Client extends BaseClient_1.BaseClient {
    constructor() {
        super();
    }
    /**
     * The Basic Auth token for the client\
     * If one does not already exist, it will be created
     */
    get basicToken() {
        return (this._basic || (this._basic = this.generateBasicToken()));
    }
    get bearerToken() {
        return (this._bearer || (this._bearer = null));
    }
    /**
     * Is the client using a bearer token to make requests?
     */
    isAuthorized() {
        return !!this.bearerToken;
    }
    /**
     * Generates a basic auth token for the iFunny API
     * @param clientId The client id for generating the basic auth token - Defaults to the client id of the client
     * @param clientSecret The client secret for generating the basic auth token - Defaults to the client secret of the client
     * @returns The basic auth token
     */
    generateBasicToken(clientId, clientSecret) {
        return (0, Methods_1.createBasicToken)(clientId !== null && clientId !== void 0 ? clientId : this.clientId, clientSecret !== null && clientSecret !== void 0 ? clientSecret : this.clientSecret);
    }
    fetch() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isAuthorized()) {
                throw new iFunnyError_1.default(ErrorCodes_1.ErrorCodes.ClientNotLoggedIn);
            }
            try {
                const response = yield this.instance.get(Endpoints_1.Endpoints.account);
                let data = response.data;
                this.payload = data.data;
                return this;
            }
            catch (error) {
                if (error instanceof Error && (0, Methods_1.isAxiosError)(error)) {
                    const resp = error.response;
                    switch ((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.status) {
                        case 401:
                            throw new iFunnyError_1.default(ErrorCodes_1.ErrorCodes.InvalidGrant, resp.data.error_description);
                    }
                }
                throw new iFunnyError_1.default(ErrorCodes_1.ErrorCodes.UnknownError, error);
            }
        });
    }
    login(email, password) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.instance.post(Endpoints_1.Endpoints.token, {
                grant_type: "password",
                data: { username: email, password },
            });
            return this;
        });
    }
}
exports.Client = Client;
//# sourceMappingURL=Client.js.map