"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.iFunnytsError = void 0;
const kCode = Symbol("code");
const ErrorCodes_1 = require("./ErrorCodes");
const Messages_1 = require("./Messages");
/**
 * A class for representing an error thrown by iFunny.ts
 */
class iFunnytsError extends Error {
    /**
     * @param code The error code for the error
     * @param args Args to pass into the message formatter function
     */
    constructor(code, ...args) {
        var _a;
        super(message(code, args));
        this[kCode] = code;
        (_a = Error.captureStackTrace) === null || _a === void 0 ? void 0 : _a.call(Error, this, iFunnytsError);
    }
    /**
     * The error code for the error
     */
    get code() {
        return this[kCode];
    }
    /**
     * The name of the error
     */
    get name() {
        return `${ErrorCodes_1.ErrorCodes[this.code]} [${this.code}]`;
    }
}
exports.iFunnytsError = iFunnytsError;
/**
 * Format the message for an error.
 * @param code The error code
 * @param args Arguments to pass for util format or as function args
 * @returns Formatted string
 */
function message(code, ...args) {
    if (typeof code !== "number")
        throw new Error("Error code must be a valid iFunnytsErrorCode");
    const msg = Messages_1.Messages[code];
    if (!msg)
        throw new Error(`An invalid error code was used: ${code}.`);
    // @ts-ignore
    if (typeof msg === "function")
        return msg(...args);
    if (!(args === null || args === void 0 ? void 0 : args.length))
        return msg;
    args.unshift(msg);
    return String(...args);
}
exports.default = iFunnytsError;
//# sourceMappingURL=iFunnyError.js.map