"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Base = void 0;
const DefaultObjectEndpoint = "/account";
/**
 * Base class for all structures.
 */
class Base {
    /**
     * @param client The client instance
     * @param payload The payload of the object
     */
    constructor(client, payload = {}) {
        /**
         * Endpoint url the object will request to
         */
        this._endpoint_url = "";
        this.client = client;
        this.payload = payload !== null && payload !== void 0 ? payload : {};
    }
    get endpoint_url() {
        return this._endpoint_url;
    }
    set endpoint_url(value) {
        this._endpoint_url = `${value}`;
    }
    get id() {
        return this.get("id");
    }
    /**
     * Base method for getting a value from the payload.
     * @param key The payload key to look up
     * @returns The payload value or null if it doesn't exist
     */
    get(key) {
        var _a;
        return (_a = this.payload[key]) !== null && _a !== void 0 ? _a : null;
    }
    /**
     * Base method for checking if a value exists in the payload.
     * @param key The payload key to look up
     * @returns Whether the payload value exists
     */
    has(key) {
        return this.payload[key] !== undefined;
    }
    /**
     * Creates a new instance of the same structure with the same payload.
     * @returns A clone of the current instance
     */
    _clone() {
        return Object.assign(Object.create(this), this);
    }
    /**
     * Updates the payload of the object.
     * @param payload The payload to merge into the current instance
     * @returns The current instance
     */
    _update_payload(payload) {
        this.payload = Object.assign(this.payload, payload);
        return this;
    }
    valueOf() {
        return this.id;
    }
}
exports.Base = Base;
//# sourceMappingURL=Base.js.map