"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAxiosError = exports.assertError = exports.assert = exports.createBasicToken = exports.OLD_createBasicToken = void 0;
const axios_1 = __importDefault(require("axios"));
const crypto_1 = __importDefault(require("crypto"));
/**
 * Certain requests have a header for 'if-modified-since' which uses the current timestamp
 * @param date Custom date
 * @returns GMT timestamp of the date
 */
function ifModifiedSince(date) {
    date !== null && date !== void 0 ? date : (date = new Date());
    return date.toUTCString();
}
/**
 * Creates a basic token for iFunny requests\
 * Basic auth tokens will be `156` characters long\
 * Basic auth tokens *should* also be made using UUIDv4
 * @param clientId The client ID for creating the basic token
 * @param clientSecret The client secret for creating the basic token
 * @returns The basic auth token
 */
function OLD_createBasicToken(clientId, clientSecret) {
    let uuid = crypto_1.default.randomUUID().replace(/\-/g, "");
    let hex = crypto_1.default.createHash("sha256").update(uuid).digest("hex").toUpperCase();
    let a = hex + `_${clientId}:`;
    let b = hex + `:${clientId}:${clientSecret}`;
    let c = crypto_1.default.createHash("sha1").update(b).digest("hex");
    return Buffer.from(a + c).toString("base64");
}
exports.OLD_createBasicToken = OLD_createBasicToken;
/**
 * Creates a basic token for iFunny requests\
 * Basic auth tokens will be `112` characters long\
 * Basic auth tokens *should* also be made using UUIDv4
 * @param clientId The client ID for creating the basic token
 * @param clientSecret The client secret for creating the basic token
 * @returns The basic auth token
 */
function createBasicToken(clientId, clientSecret) {
    let uuid = crypto_1.default.randomUUID().replace(/\-/g, "");
    let hex = uuid.toUpperCase();
    let a = hex + `_${clientId}:`;
    let b = hex + `:${clientId}:${clientSecret}`;
    let c = crypto_1.default.createHash("sha1").update(b).digest("hex");
    return Buffer.from(a + c).toString("base64");
}
exports.createBasicToken = createBasicToken;
/**
 * Asserts a claim to be true
 * @param claim The claim to check
 * @param error The message to send if the claim evaluted to false
 */
function assert(claim, error) {
    if (!claim) {
        if (typeof error === "string") {
            throw new Error(error);
        }
        else if (error instanceof Error) {
            throw error;
        }
    }
    return true;
}
exports.assert = assert;
/**
 *
 * @param item The item to check
 * @returns
 */
function assertError(item) {
    return assert(item instanceof Error, `Unknown Error: ${item}`);
}
exports.assertError = assertError;
function isAxiosError(err) {
    return axios_1.default.isAxiosError(err);
}
exports.isAxiosError = isAxiosError;
//# sourceMappingURL=Methods.js.map