import { EventEmitter } from "eventemitter3";
import axios, { AxiosInstance } from "axios";
import https from "https";
import {
	APIBase,
	APIVersion,
	DefaultClientId,
	DefaultClientSecret,
	DefaultHeaders,
	DefaultUserAgent,
} from "../utils/Constants";

import { createBasicToken } from "../utils/Methods";
import { APIClientUser as ClientPayload } from "@ifunny/ifunny-api-types/types";

export interface BaseClientConfig {
	userAgent: string;
	basic: string;
	bearer: string;
	headers: { [key: string]: string };
	clientId: string;
	clientSecret: string;
}

export type BaseClientOptions = Partial<BaseClientConfig>;

/**
 * Base Client for the Client to extend
 */
export class BaseClient extends EventEmitter {
	/**
	 * The axios instance used for requests
	 */
	public readonly instance: AxiosInstance;

	protected _payload: Partial<ClientPayload> = {};

	protected _basic: string = "";

	protected _bearer: string | null = null;

	protected clientId: string;

	protected clientSecret: string;

	constructor(options: BaseClientOptions = {}, payload: Partial<ClientPayload> = {}) {
		super();
		// Set defaults
		let headers = options?.headers ?? DefaultHeaders;
		headers["user-agent"] = options?.userAgent ?? DefaultUserAgent;
		this.instance = axios.create({
			baseURL: `https://${APIBase}/${APIVersion}`,
			headers: headers,
			httpsAgent: new https.Agent({ keepAlive: true }),
		});

		this._payload = Object.assign({}, payload);
		this.clientId = options?.clientId ?? DefaultClientId;
		this.clientSecret = options?.clientSecret ?? DefaultClientSecret;
	}

	/**
	 * Gets the payload
	 */
	get payload(): Partial<ClientPayload> {
		return this._payload;
	}

	set payload(value: Partial<ClientPayload>) {
		this._payload = Object.assign(this.payload, value);
	}

	/**
	 * Calls eval with the client set as the "this" context.
	 * @param script The script to evaluate
	 * @returns The result of the script
	 */
	protected _eval(script: string): unknown {
		return eval(script);
	}

	/**
	 * Generates a basic auth token for the iFunny API
	 * @param clientId The client id for generating the basic auth token - Defaults to the client id of the client
	 * @param clientSecret The client secret for generating the basic auth token - Defaults to the client secret of the client
	 * @returns The basic auth token
	 */
	generateBasicToken(clientId?: string, clientSecret?: string): string {
		return createBasicToken(
			clientId ?? this.clientId,
			clientSecret ?? this.clientSecret
		);
	}
}
