import axios, { AxiosError } from "axios";
import crypto from "crypto";

/**
 * Certain requests have a header for 'if-modified-since' which uses the current timestamp
 * @param date Custom date
 * @returns GMT timestamp of the date
 */
function ifModifiedSince(date?: Date): string {
	date ??= new Date();
	return date.toUTCString();
}

/**
 * Creates a basic token for iFunny requests\
 * Basic auth tokens will be `156` characters long\
 * Basic auth tokens *should* also be made using UUIDv4
 * @param clientId The client ID for creating the basic token
 * @param clientSecret The client secret for creating the basic token
 * @returns The basic auth token
 */
export function OLD_createBasicToken(clientId: string, clientSecret: string) {
	let uuid = crypto.randomUUID().replace(/\-/g, "");
	let hex = crypto.createHash("sha256").update(uuid).digest("hex").toUpperCase();
	let a = hex + `_${clientId}:`;
	let b = hex + `:${clientId}:${clientSecret}`;
	let c = crypto.createHash("sha1").update(b).digest("hex");
	return Buffer.from(a + c).toString("base64");
}

/**
 * Creates a basic token for iFunny requests\
 * Basic auth tokens will be `112` characters long\
 * Basic auth tokens *should* also be made using UUIDv4
 * @param clientId The client ID for creating the basic token
 * @param clientSecret The client secret for creating the basic token
 * @returns The basic auth token
 */
export function createBasicToken(clientId: string, clientSecret: string) {
	let uuid = crypto.randomUUID().replace(/\-/g, "");
	let hex = uuid.toUpperCase();
	let a = hex + `_${clientId}:`;
	let b = hex + `:${clientId}:${clientSecret}`;
	let c = crypto.createHash("sha1").update(b).digest("hex");
	return Buffer.from(a + c).toString("base64");
}

/**
 * Asserts a claim to be true
 * @param claim The claim to check
 * @param error The message to send if the claim evaluted to false
 */
export function assert(claim: boolean, error: Error | string): claim is true {
	if (!claim) {
		if (typeof error === "string") {
			throw new Error(error);
		} else if (error instanceof Error) {
			throw error;
		}
	}
	return true;
}

/**
 *
 * @param item The item to check
 * @returns
 */
export function assertError(item: any): item is Error {
	return assert(item instanceof Error, `Unknown Error: ${item}`);
}

export function isAxiosError(err: Error): err is AxiosError {
	return axios.isAxiosError(err);
}
